SystemOrganization addCategory: #'LookEnhancements-Extra'!

!SystemWindow class methodsFor: '*LookEnhancements' stamp: 'bvs 3/25/2004 11:05'!
initialize

	Preferences
		setPreference: #alternativeWindowBoxesLook
		toValue: false.

	"Wasn't sure how to do this with stock Monticello"
	self removeSelector: #borderWidthForRounding.
	! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/16/2004 11:13'!
addCloseBox
	"If I have a labelArea, add a close box to it"
	| frame |
	labelArea
		ifNil: [^ self].
	closeBox := self createCloseBox.
	frame := LayoutFrame new.
	frame leftFraction: 0;
		 leftOffset: 2;
		 topFraction: 0;
		 topOffset: 0.
	closeBox layoutFrame: frame.
	labelArea addMorphBack: closeBox! !

!SystemWindow methodsFor: '*LookEnhancements' stamp: 'bvs 3/24/2004 15:19'!
addCornerGrip

	| grip |
	grip _ CornerGripMorph new target: self.
	grip position: self bounds corner - grip extent.
	
	grip layoutFrame: (LayoutFrame
		fractions: (1@1 corner: 1@1) 
		offsets: (-23@-23 corner: 0@0)).
	
	self addMorphBack: grip.! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/16/2004 11:13'!
addExpandBox
	"If I have a labelArea, add a close box to it"
	| frame |
	labelArea
		ifNil: [^ self].
	expandBox := self createExpandBox.
	frame := LayoutFrame new.
	frame leftFraction: 1;
		 leftOffset: (self boxExtent x * 2 + 3) negated;
		 topFraction: 0;
		 topOffset: 0.
	expandBox layoutFrame: frame.
	labelArea addMorphBack: expandBox! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/16/2004 11:13'!
addMenuControl
	"If I have a label area, add a menu control to it."
	| frame |
	labelArea
		ifNil: [^ self].
	"No menu if no label area"
	menuBox
		ifNotNil: [menuBox delete].
	menuBox := self createMenuBox.
	frame := LayoutFrame new.
	frame leftFraction: 0;
		 leftOffset: closeBox right + 3;
		 topFraction: 0;
		 topOffset: 0.
	menuBox layoutFrame: frame.
	labelArea addMorphBack: menuBox! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 7/19/2004 10:22'!
addMorph: aMorph fullFrame: aLayoutFrame

	| left right bottom top |
	left _ aLayoutFrame leftOffset ifNil: [0].
	right _ aLayoutFrame rightOffset ifNil: [0].
	bottom _ aLayoutFrame bottomOffset ifNil: [0].
	top _ aLayoutFrame topOffset ifNil: [0].
	
	aLayoutFrame rightFraction = 1 ifTrue: [aLayoutFrame rightOffset: right - 6].
	aLayoutFrame leftOffset: left + 6.
	aLayoutFrame bottomFraction = 1 ifTrue: [aLayoutFrame bottomOffset: bottom - 6].
	aLayoutFrame topOffset: top + 6.
	aLayoutFrame topFraction = 0 ifTrue: [aLayoutFrame topOffset: top].
	
	(aMorph class name = #BrowserCommentTextMorph) ifTrue:
		[aLayoutFrame rightOffset: -6.
		aLayoutFrame leftOffset: 6.
		aLayoutFrame bottomOffset: -6.
		aLayoutFrame topOffset: 0].
	
	super addMorph: aMorph fullFrame: aLayoutFrame.

	paneMorphs _ paneMorphs copyReplaceFrom: 1 to: 0 with: (Array with: aMorph).
	Preferences alternativeWindowLook
		ifFalse:
			[aMorph borderWidth: 1.
			aMorph color: self paneColor]
		ifTrue:
			[aMorph adoptPaneColor: self paneColor.
			aMorph borderWidth: 1; borderColor: Color lightGray; color: Color white].
	Preferences scrollBarsOnRight	"reorder panes so flop-out right-side scrollbar is visible"
		ifTrue: [self addMorphBack: aMorph].
		
	self addPaneSplitters.! !

!SystemWindow methodsFor: '*LookEnhancements' stamp: 'jrp 10/2/2004 15:26'!
addPaneSplitters

	| splitters rightSplitter remainingRight remainingBottom target targetX sameX minY maxY targetY sameY minX maxX bottomSplitter |
	remainingRight _ paneMorphs copy reject: [:each | each layoutFrame rightFraction = 1].
	remainingBottom _ paneMorphs copy reject: [:each | each layoutFrame bottomFraction = 1].
	
	self removePaneSplitters.
		
	self addCornerGrip.

	[remainingRight size > 0] whileTrue:
		[target _ remainingRight first.
		targetX _ target layoutFrame rightFraction.
		sameX _ paneMorphs select: [:each | each layoutFrame rightFraction = targetX].
		minY _ (sameX detectMin: [:each | each layoutFrame topFraction]) layoutFrame topFraction.
		maxY _ (sameX detectMax: [:each | each layoutFrame bottomFraction]) layoutFrame bottomFraction.
			
		rightSplitter _ ProportionalSplitterMorph new.
		rightSplitter layoutFrame: (LayoutFrame
			fractions: (targetX @ minY corner: (targetX @ maxY))
			offsets: ((0 @ (target layoutFrame topOffset ifNil: [0]) corner: (6 @ (target layoutFrame bottomOffset ifNil: [0]))) translateBy: (target layoutFrame rightOffset ifNil: [0]) @ 0)).
		self addMorphBack: rightSplitter.
		
		remainingRight _ remainingRight copyWithoutAll: sameX].

	[remainingBottom size > 0] whileTrue:
		[target _ remainingBottom first.
		targetY _ target layoutFrame bottomFraction.
		sameY _ paneMorphs select: [:each | each layoutFrame bottomFraction = targetY].
		minX _ (sameY detectMin: [:each | each layoutFrame leftFraction]) layoutFrame leftFraction.
		maxX _ (sameY detectMax: [:each | each layoutFrame rightFraction]) layoutFrame rightFraction.
		
		bottomSplitter _ ProportionalSplitterMorph new.
		bottomSplitter beSplitsTopAndBottom.

		bottomSplitter layoutFrame: (LayoutFrame
			fractions: (minX @ targetY corner: (maxX @ targetY))
			offsets: (((target layoutFrame leftOffset ifNil: [0]) @ 0 corner: ((target layoutFrame rightOffset ifNil: [0]) @ 6)) translateBy: 0 @ (target layoutFrame bottomOffset ifNil: [0]))).
				
		self addMorphBack: bottomSplitter.
		
		remainingBottom _ remainingBottom copyWithoutAll: sameY].
	
	splitters _ self submorphsSatisfying: [:each | each isKindOf: ProportionalSplitterMorph].
	splitters do:
		[:each |
		each splitsTopAndBottom
			ifTrue:
				[self submorphsDo:
					[:eachMorph |
					(eachMorph ~= each and: [eachMorph layoutFrame bottomFraction = each layoutFrame topFraction]) ifTrue: [each addLeftOrTop: eachMorph].
					(eachMorph ~= each and: [eachMorph layoutFrame topFraction = each layoutFrame bottomFraction]) ifTrue: [each addRightOrBottom: eachMorph]]]
			ifFalse:
				[self submorphsDo:
					[:eachMorph |
					(eachMorph ~= each and: [eachMorph layoutFrame rightFraction = each layoutFrame leftFraction]) ifTrue: [each addLeftOrTop: eachMorph].
					(eachMorph ~= each and: [eachMorph layoutFrame leftFraction = each layoutFrame rightFraction]) ifTrue: [each addRightOrBottom: eachMorph]]]].
		! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 4/2/2004 16:51'!
collapseOrExpand
	"Collapse or expand the window, depending on existing state"
	| cf |
	isCollapsed
		ifTrue: 
			["Expand -- restore panes to morphics structure"
			isCollapsed _ false.
			self activate.  "Bring to frint first"
			Preferences collapseWindowsInPlace
				ifTrue: 
					[fullFrame := fullFrame align: fullFrame topLeft with: self getBoundsWithFlex topLeft]
				ifFalse:
					[collapsedFrame _ self getBoundsWithFlex].
			collapseBox ifNotNil: [collapseBox setBalloonText: 'collapse this window'].
			self setBoundsWithFlex: fullFrame.
			paneMorphs reverseDo: 
					[:m |  self addMorph: m unlock.
					self world startSteppingSubmorphsOf: m].
			self addPaneSplitters]
		ifFalse: 
			["Collapse -- remove panes from morphics structure"
			isCollapsed _ true.
			fullFrame _ self getBoundsWithFlex.
			"First save latest fullFrame"
			paneMorphs do: [:m | m delete; releaseCachedState].
			self removePaneSplitters.
			model modelSleep.
			cf := self getCollapsedFrame.
			(collapsedFrame isNil and: [Preferences collapseWindowsInPlace not]) ifTrue:
				[collapsedFrame _ cf].
			self setBoundsWithFlex: cf.
			collapseBox ifNotNil: [collapseBox setBalloonText: 'expand this window'].
			expandBox ifNotNil: [expandBox setBalloonText: 'expand to full screen']].
	self layoutChanged! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/16/2004 11:12'!
extent: aPoint 
	"Set the receiver's extent to value provided. Respect my minimumExtent."

	| newExtent |
	newExtent _ self isCollapsed
		ifTrue: [aPoint]
		ifFalse: [aPoint max: self minimumExtent].
	newExtent = self extent ifTrue: [^ self].

	isCollapsed
		ifTrue: [super extent: newExtent x @ (self labelHeight + 2)]
		ifFalse: [super extent: newExtent].
	isCollapsed
		ifTrue: [collapsedFrame _ self bounds]
		ifFalse: [fullFrame _ self bounds]! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 4/2/2004 16:55'!
getRawLabel
	| contentsFit |
	contentsFit _ label duplicate fitContents.
	contentsFit extent: (label extent x min: contentsFit extent x) @ contentsFit extent y.
	
	^ contentsFit
! !

!SystemWindow methodsFor: '*LookEnhancements' stamp: 'bvs 3/16/2004 13:20'!
gradientWithColor: aColor

	| ramp |
	ramp _ {0.0 -> Color white. 1.0 -> aColor}.

	^ (GradientFillStyle ramp: ramp)
		radial: true;
		origin: self bounds origin;
		direction: 0 @ 223;
		normal: 223 @ 0.! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/24/2004 14:48'!
initialize
	"Initialize a system window. Add label, stripes, etc., if desired"
	super initialize.
	allowReframeHandles := true.
	labelString
		ifNil: [labelString := 'Untitled Window'].
	isCollapsed := false.
	activeOnlyOnTop := true.
	paneMorphs := Array new.
	Preferences alternativeWindowLook
		ifTrue: [""
			borderColor := Color lightGray.
			borderWidth := 1.
			self color: Color lightGray lighter lighter lighter]
		ifFalse: [""
			borderColor := Color black.
			borderWidth := 1.
			color := Color black].
	self layoutPolicy: ProportionalLayout new.
	self wantsLabel
		ifTrue: [""label := StringMorph new contents: labelString;
						 font: Preferences windowTitleFont emphasis: 1.
			"Add collapse box so #labelHeight will work"
			collapseBox := self createCollapseBox.
			stripes := Array
						with: (RectangleMorph newBounds: bounds)
						with: (RectangleMorph newBounds: bounds).
			"see extent:"
			self addLabelArea.
			self setLabelWidgetAllowance.
			self addCloseBox.
			self addMenuControl.
			labelArea addMorphBack: (Morph new extent: 6 @ 0).
			labelArea addMorphBack: label.
			self wantsExpandBox
				ifTrue: [self addExpandBox].
			labelArea addMorphBack: collapseBox.
			self setFramesForLabelArea.
			Preferences clickOnLabelToEdit
				ifTrue: [label
						on: #mouseDown
						send: #relabel
						to: self].
			Preferences noviceMode
				ifTrue: [closeBox
						ifNotNil: [closeBox setBalloonText: 'close window'].
					menuBox
						ifNotNil: [menuBox setBalloonText: 'window menu'].
					collapseBox
						ifNotNil: [collapseBox setBalloonText: 'collapse/expand window']]].
				
	self addCornerGrip.
	
	self
		on: #mouseEnter
		send: #spawnReframeHandle:
		to: self.
	self
		on: #mouseLeave
		send: #spawnReframeHandle:
		to: self.
	self extent: 300 @ 200.
	mustNotClose := false.
	updatablePanes := Array new! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 4/2/2004 16:48'!
labelHeight
	"Answer the height for the window label.  The standard behavior is at bottom; a hook is provided so that models can stipulate other heights, in support of various less-window-looking demos."

	| aHeight |
	(model notNil and: [model respondsTo: #desiredWindowLabelHeightIn:]) ifTrue:
		[(aHeight _ model desiredWindowLabelHeightIn: self) ifNotNil: [^ aHeight]].

	^ label ifNil: [0] ifNotNil:
		 [(label height + 12) max:
			(collapseBox ifNotNil: [collapseBox height] ifNil: [10])]! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bmk 3/19/2002 02:11'!
mouseMove: evt
	"Handle a mouse-move event"

	| cp |
	cp _ evt cursorPoint.
	self valueOfProperty: #clickPoint ifPresentDo: 
		[:firstClick |
		((self labelRect containsPoint: firstClick) and: [(cp dist: firstClick) > 3]) ifTrue:
		["If this is a drag that started in the title bar, then pick me up"
		^ self isSticky ifFalse:
			[self fastFramingOn 
				ifTrue: [self doFastFrameDrag: firstClick]
				ifFalse: [evt hand grabMorph: self topRendererOrSelf]]]].
	model windowActiveOnFirstClick ifTrue:
		["Normally window takes control on first click.
		Need explicit transmission for first-click activity."
		submorphs do: [:m | (m containsPoint: cp) ifTrue: [m mouseMove: evt]]]! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/17/2004 09:15'!
paneColor: aColor
	self setProperty: #paneColor toValue: aColor.

	self adoptPaneColor: aColor.! !

!SystemWindow methodsFor: '*LookEnhancements' stamp: 'bvs 4/2/2004 16:49'!
removePaneSplitters

	| splitters corners |
	splitters _ self submorphsSatisfying: [:each | each isKindOf: ProportionalSplitterMorph].
	splitters do: [:each | each delete].
	
	corners _ self submorphsSatisfying: [:each | each isKindOf: CornerGripMorph].
	corners do: [:each | each delete].
! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/16/2004 12:22'!
setFramesForLabelArea
	"an aid to converting old instances, but then I found  
	convertAlignment (jesse welton's note)"
	| frame |
	labelArea
		ifNil: [^ self].
		
	labelArea
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		layoutInset: 0.
	label hResizing: #spaceFill.
	
	labelArea
		ifNotNil: [frame := LayoutFrame new.
			frame leftFraction: 0;
				 topFraction: 0;
				 rightFraction: 1;
				 topOffset: self labelHeight negated.
			Preferences alternativeWindowLook
				ifTrue: [frame leftOffset: 6;
						 rightOffset: -6;
						 topOffset: self labelHeight negated + 6;
						 bottomOffset: -6].
			labelArea layoutFrame: frame]! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'jlr 6/18/2005 12:11'!
setStripeColorsFrom: paneColor 
	"Set the stripe color based on the given paneColor"

	labelArea ifNotNil: 
			[""

			labelArea color: (Preferences alternativeWindowLook 
						ifTrue: [Color transparent]
						ifFalse: [Color transparent])].
	""
	self updateBoxesColor: (self isActive 
				ifTrue: [paneColor]
				ifFalse: [paneColor muchDarker]).
	""
	stripes ifNil: [^self].
	""
	Preferences alternativeWindowLook 
		ifTrue: 
			[""

			self isActive 
				ifTrue: 
					[""

					self fillStyle: (self gradientWithColor: paneColor lighter lighter )]
				ifFalse: 
					["This could be much faster"

					self fillStyle: (self gradientWithColor: paneColor lighter lighter lighter)]]
		ifFalse: 
			[""

			self isActive 
				ifTrue: 
					[""

					(stripes second)
						color: paneColor;
						borderColor: stripes second color darker.
					(stripes first)
						color: stripes second borderColor darker;
						borderColor: stripes first color darker]
				ifFalse: 
					["This could be much faster"

					(stripes second)
						color: paneColor;
						borderColor: paneColor.
					(stripes first)
						color: paneColor;
						borderColor: paneColor]]! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/24/2004 15:27'!
spawnPaneFrameHandle: event 
	| resizer localPt side growingPane newBounds adjoiningPanes limit cursor |
	true ifTrue: [^ self]. "Working on new resizers"
	
	(self world firstSubmorph isKindOf: NewHandleMorph) 
		ifTrue: [^self	"Prevent multiple handles"].
	((self innerBounds withHeight: self labelHeight + 4) 
		containsPoint: event cursorPoint) 
			ifTrue: [^self	"in label or top of top pane"].
	growingPane := self paneWithLongestSide: [:s | side := s]
				near: event cursorPoint.
	growingPane ifNil: [^self].
	"don't resize pane side coincident with window side - RAA 5 jul 2000"
	(growingPane perform: side) = (self innerBounds perform: side) 
		ifTrue: [^self].
	(side == #top and: [growingPane top = self panelRect top]) ifTrue: [^self].
	adjoiningPanes := paneMorphs 
				select: [:pane | pane bounds bordersOn: growingPane bounds along: side].
	limit := adjoiningPanes isEmpty 
				ifFalse: 
					[(adjoiningPanes collect: [:pane | pane bounds perform: side]) 
						perform: ((#(#top #left) includes: side) ifTrue: [#max] ifFalse: [#min])]
				ifTrue: [self bounds perform: side].
	cursor := Cursor resizeForEdge: side.
	resizer := (NewHandleMorph new)
				sensorMode: self fastFramingOn;
				followHand: event hand
					forEachPointDo: 
						[:p | 
						localPt := self pointFromWorld: p.
						newBounds := growingPane bounds 
									withSideOrCorner: side
									setToPoint: localPt
									minExtent: 40 @ 20
									limit: limit.
						self fastFramingOn 
							ifTrue: 
								["For fast display, only higlight the rectangle during loop"

								Cursor currentCursor == cursor 
									ifFalse: 
										[(event hand)
											visible: false;
											refreshWorld;
											visible: true.
										cursor show].
								newBounds := growingPane bounds newRectButtonPressedDo: 
												[:f | 
												growingPane bounds 
													withSideOrCorner: side
													setToPoint: (self pointFromWorld: Sensor cursorPoint)
													minExtent: 40 @ 20
													limit: limit].].
								self 
									reframePanesAdjoining: growingPane
									along: side
									to: newBounds.
]
					lastPointDo: [:p | ]
					withCursor: cursor.
	event hand world addMorphInLayer: resizer.
	resizer startStepping! !

!SystemWindow methodsFor: '*LookEnhancements-override' stamp: 'bvs 3/24/2004 15:20'!
spawnReframeHandle: event
	"The mouse has crossed a pane border.  Spawn a reframe handle."
	| resizer localPt pt ptName newBounds cursor |
	allowReframeHandles ifFalse: [^ self].
	owner ifNil: [^ self  "Spurious mouseLeave due to delete"].
	(self isActive not or: [self isCollapsed]) ifTrue:  [^ self].
	((self world ifNil: [^ self]) firstSubmorph isKindOf: NewHandleMorph) ifTrue:
		[^ self  "Prevent multiple handles"].
	pt _ event cursorPoint.
	"prevent spurios mouse leave when dropping morphs"
	owner morphsInFrontOf: self overlapping: (pt-2 extent: 4@4)
		do:[:m| m isHandMorph ifFalse:[(m fullContainsPoint: pt) ifTrue:[^self]]].
	ptName ifNil:
		["Check for pane border adjust"
		^ self spawnPaneFrameHandle: event].

	cursor _ Cursor resizeForEdge: ptName.
	resizer _ NewHandleMorph new
		sensorMode: self fastFramingOn;

		followHand: event hand
		forEachPointDo:
			[:p | localPt _ self pointFromWorld: p.
			newBounds _ self bounds
				withSideOrCorner: ptName
				setToPoint: localPt
				minExtent: self minimumExtent.
			self fastFramingOn 
			ifTrue:
				[Cursor currentCursor == cursor ifFalse:[
					event hand visible: false; refreshWorld; visible: true.
					cursor show].
				self doFastWindowReframe: ptName]
			ifFalse:
				[self bounds: newBounds.
				(Preferences roundedWindowCorners
					and: [#(bottom right bottomRight) includes: ptName])
					ifTrue:
					["Complete kluge: causes rounded corners to get painted correctly,
					in spite of not working with top-down displayWorld."
					ptName = #bottom ifFalse:
						[self invalidRect: (self bounds topRight - (6@0) extent: 7@7)].
					ptName = #right ifFalse:
						[self invalidRect: (self bounds bottomLeft - (0@6) extent: 7@7)].
					self invalidRect: (self bounds bottomRight - (6@6) extent: 7@7)]]]
		lastPointDo:
			[:p | ]
		withCursor: cursor.
	event hand world addMorphInLayer: resizer.
	resizer startStepping! !

!MenuMorph class methodsFor: '*LookEnhancements' stamp: 'jrp 11/3/2004 12:05'!
initialize

	"MenuMorph initialize"
	
	Preferences
		setParameter: #menuTitleBorderWidth to: 0;
		setParameter: #menuTitleColor to: (TranslucentColor r: 0.87 g: 0.8 b: 1 alpha: 0.65);
		setParameter: #menuColor to: (Color 
			r: (215/255) asFloat 
			g: (220/255) asFloat 
			b: (232/255) asFloat)
! !

!MenuMorph methodsFor: '*LookEnhancements-override' stamp: 'jrp 10/23/2004 16:20'!
addStayUpIcons
	| title closeBox pinBox titleBarArea |
	title _ submorphs
				detect: [:ea | ea hasProperty: #titleString]
				ifNone: [self setProperty: #needsTitlebarWidgets toValue: true.
					^ self].
	closeBox _ IconicButton new target: self;
				 actionSelector: #delete;
				 labelGraphic: self class closeBoxImage;
				 color: Color transparent;
				 extent: 14 @ 16;
				 borderWidth: 0.
	pinBox _ IconicButton new target: self;
				 actionSelector: #stayUp:;
				 arguments: {true};
				 labelGraphic: self class pushPinImage;
				 color: Color transparent;
				 extent: 14 @ 15;
				 borderWidth: 0.
	Preferences noviceMode
		ifTrue: [closeBox setBalloonText: 'close this menu'.
			pinBox setBalloonText: 'keep this menu up'].
	titleBarArea _  AlignmentMorph newRow vResizing: #shrinkWrap;
			 layoutInset: 3;
			 color: Preferences menuTitleColor;
			 addMorphBack: closeBox;
			 addMorphBack: title;
			 addMorphBack: pinBox.
	
	title color: Color transparent.
	title firstSubmorph font: Preferences windowTitleFont.
	Preferences roundedMenuCorners
		ifTrue: [titleBarArea useRoundedCorners].
	
	self addMorphFront: titleBarArea.
	titleBarArea setProperty: #titleString toValue: (title valueOfProperty: #titleString).
	title removeProperty: #titleString.
	self setProperty: #hasTitlebarWidgets toValue: true.
	self removeProperty: #needsTitlebarWidgets.
	self removeStayUpItems! !

!MenuMorph methodsFor: '*LookEnhancements-override' stamp: 'jrp 10/23/2004 15:10'!
addTitle: aString updatingSelector: aSelector updateTarget: aTarget 
	"Add a title line at the top of this menu Make aString its initial 
	contents.  
	If aSelector is not nil, then periodically obtain fresh values for its 
	contents by sending aSelector to aTarget.."
	| title |
	title _ AlignmentMorph new.
	self setTitleParametersFor: title.
	title vResizing: #shrinkWrap.
	title listDirection: #topToBottom.
	title wrapCentering: #center;
		 cellPositioning: #topCenter;

		 layoutInset: 0.
	aSelector
		ifNil: [(aString asString findTokens: String cr)
				do: [:line | title addMorphBack: (StringMorph new contents: line;
							 font: Preferences windowTitleFont)]]
		ifNotNil: [title addMorphBack: (UpdatingStringMorph new lock; font: Preferences windowTitleFont; useStringFormat; target: aTarget; getSelector: aSelector)].
	title setProperty: #titleString toValue: aString.
	self addMorphFront: title.
	(self hasProperty: #needsTitlebarWidgets)
		ifTrue: [self addStayUpIcons]! !

!MenuMorph methodsFor: '*LookEnhancements-override' stamp: 'jrp 10/23/2004 15:13'!
setTitleParametersFor: aMenuTitle 
	| menuTitleColor menuTitleBorderColor |
	Preferences roundedMenuCorners
		ifTrue: [aMenuTitle useRoundedCorners].
	""
	menuTitleColor _ Preferences menuColorFromWorld
				ifTrue: [self color darker]
				ifFalse: [Preferences menuTitleColor].
	""
	menuTitleBorderColor _ Preferences menuAppearance3d
				ifTrue: [#inset]
				ifFalse: [Preferences menuColorFromWorld
						ifTrue: [self color darker muchDarker]
						ifFalse: [Preferences menuTitleBorderColor]].
	""
	aMenuTitle
		setColor: menuTitleColor
		borderWidth: Preferences menuTitleBorderWidth
		borderColor: menuTitleBorderColor! !

!MenuMorph methodsFor: '*LookEnhancements-override' stamp: 'jrp 11/3/2004 12:04'!
updateColor
	| fill title |
	Preferences gradientMenu
		ifFalse: [^ self].
	""
	fill _ GradientFillStyle ramp: {0.0 -> Color white. 1 -> self color}.
	""
	fill radial: false.
	fill origin: self topLeft.
	fill direction: 0 @ self height.
	""
	self fillStyle: fill.
	" 
	update the title color"
	title _ self allMorphs
				detect: [:each | each hasProperty: #titleString]
				ifNone: [^ self].
	""
	fill _ GradientFillStyle ramp: {0.0 -> title color twiceLighter. 1 -> title color twiceDarker}.
	""
	fill origin: title topLeft.
	fill direction: title width @ 0.
	""
	title fillStyle: fill! !

Morph subclass: #CornerGripMorph
	instanceVariableNames: 'target lastMouse'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LookEnhancements-Extra'!

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 15:36'!
drawOn: aCanvas

	| dotBounds |
	super drawOn: aCanvas.
	
	bounds _ self bounds.
		
	dotBounds _ Rectangle
		center: bounds corner - (16 @ 3)
		extent: 4 @ 4.
	aCanvas
		fillRectangle: dotBounds
		color: Color lightGray.
		
	dotBounds _ Rectangle
		center: bounds corner - (3 @ 16)
		extent: 4 @ 4.
	aCanvas
		fillRectangle: dotBounds
		color: Color lightGray.
! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 14:36'!
handlesMouseDown: anEvent

	^ true! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 14:36'!
handlesMouseOver: anEvent

	^ true! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 15:14'!
initialize

	super initialize.
	
	self color: Color transparent.
	self extent: 24 @ 24.
	! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 14:36'!
mouseDown: anEvent

	lastMouse _ anEvent cursorPoint! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 14:36'!
mouseEnter: anEvent

	anEvent hand showTemporaryCursor: self resizeCursor! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 14:37'!
mouseLeave: anEvent

	anEvent hand showTemporaryCursor: nil! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 14:43'!
mouseMove: anEvent

	| delta oldBounds |
	target ifNil: [^ self].
	
	delta _ anEvent cursorPoint - lastMouse.
	lastMouse _ anEvent cursorPoint.
	
	oldBounds _ target bounds.
	target bounds: (oldBounds origin corner: (oldBounds corner + delta)).
	
	self bounds: (self bounds origin + delta extent: self bounds extent).! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 14:39'!
resizeCursor

	^ Cursor resizeForEdge: #bottomRight! !

!CornerGripMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 14:38'!
target: aMorph

	target _ aMorph! !

Morph subclass: #ProportionalSplitterMorph
	instanceVariableNames: 'leftOrTop rightOrBottom splitsTopAndBottom lastMouse'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LookEnhancements-Extra'!

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:57'!
addLeftOrTop: aMorph

	leftOrTop add: aMorph! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:55'!
addRightOrBottom: aMorph

	rightOrBottom add: aMorph.
	
	! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
beSplitsTopAndBottom

	splitsTopAndBottom _ true.
	! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
drawOn: aCanvas

	| dotBounds |
	super drawOn: aCanvas.
	
	dotBounds _ Rectangle
		center: self bounds center
		extent: 4 @ 4.
	aCanvas
		fillRectangle: dotBounds
		color: Color lightGray
! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
handlesMouseDown: anEvent

	^ true! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
handlesMouseOver: anEvent

	^ true
	! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 17:16'!
initialize

	super initialize.
	
	self color: Color transparent.
	self hResizing: #spaceFill.
	self vResizing: #spaceFill.
	splitsTopAndBottom _ false.
	
	leftOrTop _ OrderedCollection new.
	rightOrBottom _ OrderedCollection new.! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
mouseDown: anEvent

	lastMouse _ anEvent cursorPoint! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
mouseEnter: anEvent

	anEvent hand showTemporaryCursor: self resizeCursor! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
mouseLeave: anEvent

	anEvent hand showTemporaryCursor: nil! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 17:28'!
mouseMove: anEvent

	| delta firstRight firstBottom secondLeft secondTop selfTop selfBottom selfLeft selfRight |
		
	delta _ splitsTopAndBottom
		ifTrue: [0 @ (anEvent cursorPoint y - lastMouse y)]
		ifFalse: [(anEvent cursorPoint x - lastMouse x) @ 0].
	lastMouse _ anEvent cursorPoint.
	
	leftOrTop do:
		[:each |
		firstRight _ each layoutFrame rightOffset ifNil: [0].
		firstBottom _ each layoutFrame bottomOffset ifNil: [0].
		each layoutFrame rightOffset: firstRight + delta x.
		each layoutFrame bottomOffset: firstBottom + delta y].
	
	rightOrBottom do:
		[:each |
		secondLeft _ each layoutFrame leftOffset ifNil: [0].
		secondTop _ each layoutFrame topOffset ifNil: [0].
		each layoutFrame leftOffset: secondLeft + delta x.
		each layoutFrame topOffset: secondTop + delta y].
	
	selfTop _ self layoutFrame topOffset ifNil: [0].
	selfBottom _ self layoutFrame bottomOffset ifNil: [0].
	selfLeft _ self layoutFrame leftOffset ifNil: [0].
	selfRight _ self layoutFrame rightOffset ifNil: [0].
	
	self layoutFrame topOffset: selfTop + delta y.
	self layoutFrame bottomOffset: selfBottom + delta y.
	self layoutFrame leftOffset: selfLeft + delta x.
	self layoutFrame rightOffset: selfRight + delta x.
	
	self owner layoutChanged.! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'jrp 6/8/2004 21:42'!
mouseUp: anEvent

	(self bounds containsPoint: anEvent cursorPoint) ifFalse: 
		[anEvent hand showTemporaryCursor: nil]! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
resizeCursor

	^ Cursor resizeForEdge: (splitsTopAndBottom
		ifTrue: [#top]
		ifFalse: [#left])
		! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 17:25'!
splitsTopAndBottom

	^ splitsTopAndBottom! !

!ProportionalSplitterMorph methodsFor: 'as yet unclassified' stamp: 'bvs 3/24/2004 16:39'!
wantsEveryMouseMove

	^ true! !

!BrowserCommentTextMorph methodsFor: '*LookEnhancements-override' stamp: 'jrp 10/3/2004 18:40'!
hidePane
	| win |
	self window ifNotNilDo: [:window | window removePaneSplitters].
	
	self lowerPane ifNotNilDo:
		[ :lp | 
		lp layoutFrame bottomFraction: self layoutFrame bottomFraction.
		lp layoutFrame bottomOffset: -6].
	win _ self window ifNil: [ ^self ].
	self delete.
	win updatePanesFromSubmorphs.
	win addPaneSplitters! !

!BrowserCommentTextMorph methodsFor: '*LookEnhancements-override' stamp: 'jrp 10/3/2004 18:39'!
showPane
	owner ifNil: [
		| win |
		win _ self window ifNil: [ ^self ].
		win addMorph: self fullFrame: self layoutFrame.
		win updatePanesFromSubmorphs ].

	self lowerPane ifNotNilDo: [ :lp | lp layoutFrame bottomFraction: self layoutFrame topFraction ].
	
	self window ifNotNilDo: [:win | win addPaneSplitters]! !
SystemWindow initialize!
MenuMorph initialize!
